/**************************************************************************
 **                                                                      **
 ** Copyright (C) 2011-2020 Lukas Spies                                  **
 ** Contact: http://photoqt.org                                          **
 **                                                                      **
 ** This file is part of PhotoQt.                                        **
 **                                                                      **
 ** PhotoQt is free software: you can redistribute it and/or modify      **
 ** it under the terms of the GNU General Public License as published by **
 ** the Free Software Foundation, either version 2 of the License, or    **
 ** (at your option) any later version.                                  **
 **                                                                      **
 ** PhotoQt is distributed in the hope that it will be useful,           **
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of       **
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        **
 ** GNU General Public License for more details.                         **
 **                                                                      **
 ** You should have received a copy of the GNU General Public License    **
 ** along with PhotoQt. If not, see <http://www.gnu.org/licenses/>.      **
 **                                                                      **
 **************************************************************************/

#include "handlingfilemanagement.h"

bool PQHandlingFileManagement::renameFile(QString dir, QString oldName, QString newName) {

    DBG << CURDATE << "PQHandlingFileManagement::renameFile()" << NL
        << CURDATE << "** dir = " << dir.toStdString() << NL
        << CURDATE << "** oldName = " << oldName.toStdString() << NL
        << CURDATE << "** newName = " << newName.toStdString() << NL;

    QFile file(dir + "/" + oldName);
    return file.rename(dir + "/" + newName);

}

bool PQHandlingFileManagement::deleteFile(QString filename, bool permanent) {

    DBG << CURDATE << "PQHandlingFileManagement::deleteFile()" << NL
        << CURDATE << "** filename = " << filename.toStdString() << NL
        << CURDATE << "** permanent = " << permanent << NL;

#ifndef Q_OS_WIN
    if(permanent) {
#endif

        QFile file(filename);
        return file.remove();

#ifndef Q_OS_WIN
    } else {

        // The file to delete
        QFile file(filename);

        // Of course we only proceed if the file actually exists
        if(file.exists()) {

            // Create the meta .trashinfo file
            QString info = "[Trash Info]\n";
            info += "Path=" + QUrl(filename).toEncoded() + "\n";
            info += "DeletionDate=" + QDateTime::currentDateTime().toString("yyyy-MM-ddThh:mm:ss");

            // The base patzh for the Trah (files on external devices  use the external device for Trash)
            QString baseTrash = "";

            // If file lies in the home directory
            if(QFileInfo(filename).absoluteFilePath().startsWith(QDir::homePath())) {

                // Set the base path and make sure all the dirs exist
                baseTrash = ConfigFiles::GENERIC_DATA_DIR() + "/Trash/";

                QDir dir;
                dir.setPath(baseTrash);
                if(!dir.exists()) {
                    if(!dir.mkpath(baseTrash)) {
                        LOG << "PQHandlingFileManagement [mkdir home: baseTrash] ERROR: mkdir() failed!";
                        return false;
                    }
                }
                dir.setPath(baseTrash + "files");
                if(!dir.exists()) {
                    if(!dir.mkdir(baseTrash + "files")) {
                        LOG << "PQHandlingFileManagement [mkdir home: baseTrash/Trash] ERROR: mkdir() failed!";
                        return false;
                    }
                }
                dir.setPath(baseTrash + "info");
                if(!dir.exists()) {
                    if(!dir.mkdir(baseTrash + "info")) {
                        LOG << "PQHandlingFileManagement [mkdir home: baseTrash/info] ERROR: mkdir() failed!";
                        return false;
                    }
                }
            } else {
                // Set the base path ...
                for(QStorageInfo &storage : QStorageInfo::mountedVolumes()) {
                    if(!storage.isReadOnly() && storage.isValid() && filename.startsWith(storage.rootPath()) &&
                       baseTrash.length() < storage.rootPath().length()) {
                        baseTrash = storage.rootPath();
                    }
                }
                baseTrash += "/" + QString("/.Trash-%1/").arg(getuid());
                // ... and make sure all the dirs exist
                QDir dir;
                dir.setPath(baseTrash);
                if(!dir.exists()) {
                    if(!dir.mkdir(baseTrash)) {
                        LOG << "PQHandlingFileManagement [mkdir baseTrash] ERROR: mkdir() failed!";
                        return false;
                    }
                }
                dir.setPath(baseTrash + "files");
                if(!dir.exists()) {
                    if(!dir.mkdir(baseTrash + "files")) {
                        LOG << "PQHandlingFileManagement [mkdir baseTrash/files] ERROR: mkdir() failed!";
                        return false;
                    }
                }
                dir.setPath(baseTrash + "info");
                if(!dir.exists()) {
                    if(!dir.mkdir(baseTrash + "info")) {
                        LOG << "PQHandlingFileManagement [mkdir baseTrash/info] ERROR: mkdir() failed!";
                        return false;
                    }
                }

            }

            // that's the new trash file
            QString trashFile = baseTrash + "files/" + QUrl::toPercentEncoding(QFileInfo(file).fileName(),""," ");
            QString backupTrashFile = trashFile;

            // If there exists already a file with that name, we simply append the next higher number (sarting at 1)
            QFile ensure(trashFile);
            int j = 1;
            while(ensure.exists()) {
                trashFile = backupTrashFile + QString(" (%1)").arg(j++);
                ensure.setFileName(trashFile);
            }

            // Copy the file to the Trash
            if(file.copy(trashFile)) {

                // And remove the old file
                if(!file.remove()) {
                    LOG << CURDATE << "PQHandlingFileManagement: ERROR: Old file couldn't be removed!" << NL;
                    return false;
                }

                // Write the .trashinfo file
                QFile i(baseTrash + "info/" + QFileInfo(trashFile).fileName() + ".trashinfo");
                if(i.open(QIODevice::WriteOnly)) {
                    QTextStream out(&i);
                    out << info;
                    i.close();
                } else {
                    LOG << CURDATE << "PQHandlingFileManagement: ERROR: *.trashinfo file couldn't be created!" << NL;
                    return false;
                }

            } else {
                LOG << CURDATE << "PQHandlingFileManagement: ERROR: File couldn't be deleted (moving file failed)" << NL;
                return false;
            }

        } else {
            LOG << CURDATE << "PQHandlingFileManagement: ERROR: File '" << filename.toStdString() << "' doesn't exist...?" << NL;
            return false;
        }

    }

    return true;

#endif

}

QString PQHandlingFileManagement::copyFile(QString filename) {

    DBG << CURDATE << "PQHandlingFileManagement::copyFile()" << NL
        << CURDATE << "** filename = " << filename.toStdString() << NL;

    QString ending = QFileInfo(filename).suffix();

    //: Title of filedialog to select new filename/location to copy file to.
    QString newfilename = QFileDialog::getSaveFileName(0, "Where to copy the file to", filename, QString("*.%1 (*.%2)").arg(ending).arg(ending));

    if(newfilename.trimmed() == "")
        return "";

    QFile file(filename);
    if(!file.copy(newfilename)) {
        LOG << CURDATE << "PQHandlingFileManagement::moveFile(): ERROR: The file could not be copied to its new location." << NL;
        return "";
    }

    return newfilename;

}

QString PQHandlingFileManagement::moveFile(QString filename) {

    DBG << CURDATE << "PQHandlingFileManagement::moveFile()" << NL
        << CURDATE << "** filename = " << filename.toStdString() << NL;

    QString ending = QFileInfo(filename).suffix();

    //: Title of filedialog to select new filename/location to move file to.
    QString newfilename = QFileDialog::getSaveFileName(0, "Where to move the file to", filename, QString("*.%1 (*.%2)").arg(ending).arg(ending));

    if(newfilename.trimmed() == "")
        return "";

    QFile file(filename);
    if(!file.copy(newfilename)) {
        LOG << CURDATE << "PQHandlingFileManagement::moveFile(): ERROR: The file could not be moved to its new location, copy process failed." << NL;
        return "";
    }

    if(!file.remove()) {
        LOG << CURDATE << "PQHandlingFileManagement::moveFile(): ERROR: The file was successfully copied to new location but the old file could not be removed." << NL;
        return newfilename;
    }

    return newfilename;

}
